unit Main;

interface
uses
  Messages, WinTypes, Classes, Graphics,
  Forms, Dialogs, StdCtrls, ExtCtrls, Controls;

type
  TForm1 = class(TForm)
    Image1: TImage;
    Panel1: TPanel;
    Button1: TButton;
    Button2: TButton;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
  private
    { Private declarations }
    PictureDLL: THandle;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

{ Utility routines }
procedure LoadDLLBmp(Lib:THandle; BmpId:Integer; Dest:TBitmap);
function  LoadDLLStr(Lib:THandle; StrId:Integer): String;


implementation
uses SysUtils,WinProcs;

{$R *.DFM}
{$I pics.inc}  { resource declarations }

const
 Err1 = 'Unable to load %s (%d)';
 Err2 = 'Resource #%d not found (%s)';
 Err3 = 'Bitmap #%d hasn''t been stored properly';
 PictDLLName = 'Pictures.dll';


procedure TForm1.FormCreate(Sender: TObject);
begin
  PictureDLL := LoadLibrary(PictDLLName);
  if PictureDLL < HINSTANCE_ERROR then
    raise Exception.CreateFmt(Err1,[PictDLLName,PictureDLL]);
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  if PictureDLL <> 0 then FreeLibrary(PictureDLL);
end;


procedure TForm1.Button1Click(Sender: TObject);
begin
  LoadDLLBmp(PictureDLL, BChemical, Image1.Picture.Bitmap);
  Caption := LoadDLLStr(PictureDLL,SDescr1);
end;

procedure TForm1.Button2Click(Sender: TObject);
begin
  LoadDLLBmp(PictureDLL, BShipping, Image1.Picture.Bitmap);
  Caption := LoadDLLStr(PictureDLL,SDescr2);
end;


procedure LoadDLLBmp(Lib:THandle; BmpId:Integer; Dest:TBitmap);
{ Based on palette-preserving code in Lloyd's LDELPHI.HLP }
var
  rInfo,hMemory: THandle;
  rSize: Longint;
  pData: PByte;
  BmpHdr: TBitmapFileHeader;
  stream: TMemoryStream;
begin
  if Lib < HINSTANCE_ERROR then Exit;
  { locate bitmap within library }
  rInfo := FindResource(Lib, MakeIntResource(BmpId), rt_Bitmap);
  if rInfo = 0 then raise Exception.CreateFmt(Err2,[BmpId,'FindResource']);
  { load bitmap into global memory }
  hMemory := LoadResource(Lib,rInfo);
  try
    if hMemory = 0 then raise Exception.CreateFmt(Err2,[BmpId,'LoadResource']);
    { pin down the global memory to a specific address }
    pData := LockResource(hMemory);
    try
      { find upperbound of bitmap library usage }
      rSize := SizeofResource(Lib,rInfo);
      if rSize = 0 then raise Exception.CreateFmt(Err3,[BmpId]);
      { write to a stream so we can use TBitmap.LoadFromStream }
      stream := TMemoryStream.Create;
      try
        { fake a BM header to keep LoadFromStream happy }
        BmpHdr.bfType := $4D42;
        stream.SetSize(sizeof(BmpHdr)+rSize);
        stream.Write(BmpHdr,sizeof(BmpHdr));
        stream.Write(pData^,rSize);
        stream.Seek(0,0);
        Dest.LoadFromStream(stream);
      finally
        stream.Free;
      end;
    finally
      UnlockResource(hMemory);
    end;
  finally
    FreeResource(hMemory);
  end;
end;

function LoadDLLStr(Lib:THandle; StrId:Integer):String;
var a: array[0..254] of char;
begin
  result := '';
  if Lib < HINSTANCE_ERROR then Exit;
  if LoadString(Lib,StrId,a,sizeof(a)) > 0 then
    result := StrPas(a);
end;


end.
